@@ -15,10 +15,35 @@ pub trait Description {
     fn description() -> &'static str;
 }
 
-/// Represents a GraphQL type.
-///
-/// All GraphQL types implement this trait, such as `Scalar`, `Object`, `Union` ...
-pub trait Type {
+/// Represents a GraphQL input type.
+pub trait InputType: Send + Sync + Sized {
+    /// Type the name.
+    fn type_name() -> Cow<'static, str>;
+
+    /// Qualified typename.
+    fn qualified_type_name() -> String {
+        format!("{}!", Self::type_name())
+    }
+
+    /// Create type information in the registry and return qualified typename.
+    fn create_type_info(registry: &mut registry::Registry) -> String;
+
+    /// Parse from `Value`. None represents undefined.
+    fn parse(value: Option<Value>) -> InputValueResult<Self>;
+
+    /// Convert to a `Value` for introspection.
+    fn to_value(&self) -> Value;
+
+    /// Get the federation fields, only for InputObject.
+    #[doc(hidden)]
+    fn federation_fields() -> Option<String> {
+        None
+    }
+}
+
+/// Represents a GraphQL output type.
+#[async_trait::async_trait]
+pub trait OutputType: Send + Sync {
     /// Type the name.
     fn type_name() -> Cow<'static, str>;
 
@@ -36,26 +61,7 @@ pub trait Type {
 
     /// Create type information in the registry and return qualified typename.
     fn create_type_info(registry: &mut registry::Registry) -> String;
-}
 
-/// Represents a GraphQL input value.
-pub trait InputType: Type + Send + Sync + Sized {
-    /// Parse from `Value`. None represents undefined.
-    fn parse(value: Option<Value>) -> InputValueResult<Self>;
-
-    /// Convert to a `Value` for introspection.
-    fn to_value(&self) -> Value;
-
-    /// Get the federation fields, only for InputObject.
-    #[doc(hidden)]
-    fn federation_fields() -> Option<String> {
-        None
-    }
-}
-
-/// Represents a GraphQL output value.
-#[async_trait::async_trait]
-pub trait OutputType: Type + Send + Sync {
     /// Resolve an output value to `async_graphql::Value`.
     async fn resolve(
         &self,
@@ -64,7 +70,8 @@ pub trait OutputType: Type + Send + Sync {
     ) -> ServerResult<Value>;
 }
 
-impl<T: Type + ?Sized> Type for &T {
+#[async_trait::async_trait]
+impl<T: OutputType + ?Sized> OutputType for &T {
     fn type_name() -> Cow<'static, str> {
         T::type_name()
     }
@@ -72,10 +79,7 @@ impl<T: Type + ?Sized> Type for &T {
     fn create_type_info(registry: &mut Registry) -> String {
         T::create_type_info(registry)
     }
-}
 
-#[async_trait::async_trait]
-impl<T: OutputType + ?Sized> OutputType for &T {
     #[allow(clippy::trivially_copy_pass_by_ref)]
     async fn resolve(
         &self,
@@ -86,22 +90,16 @@ impl<T: OutputType + ?Sized> OutputType for &T {
     }
 }
 
-impl<T: Type, E: Into<Error> + Send + Sync + Clone> Type for Result<T, E> {
+#[async_trait::async_trait]
+impl<T: OutputType + Sync, E: Into<Error> + Send + Sync + Clone> OutputType for Result<T, E> {
     fn type_name() -> Cow<'static, str> {
         T::type_name()
     }
 
-    fn qualified_type_name() -> String {
-        T::qualified_type_name()
-    }
-
-    fn create_type_info(registry: &mut registry::Registry) -> String {
+    fn create_type_info(registry: &mut Registry) -> String {
         T::create_type_info(registry)
     }
-}
 
-#[async_trait::async_trait]
-impl<T: OutputType + Sync, E: Into<Error> + Send + Sync + Clone> OutputType for Result<T, E> {
     async fn resolve(
         &self,
         ctx: &ContextSelectionSet<'_>,
@@ -131,7 +129,8 @@ pub trait UnionType: ContainerType {}
 /// A GraphQL input object.
 pub trait InputObjectType: InputType {}
 
-impl<T: Type + ?Sized> Type for Box<T> {
+#[async_trait::async_trait]
+impl<T: OutputType + ?Sized> OutputType for Box<T> {
     fn type_name() -> Cow<'static, str> {
         T::type_name()
     }
@@ -139,10 +138,7 @@ impl<T: Type + ?Sized> Type for Box<T> {
     fn create_type_info(registry: &mut Registry) -> String {
         T::create_type_info(registry)
     }
-}
 
-#[async_trait::async_trait]
-impl<T: OutputType + ?Sized> OutputType for Box<T> {
     #[allow(clippy::trivially_copy_pass_by_ref)]
     async fn resolve(
         &self,
@@ -155,6 +151,14 @@ impl<T: OutputType + ?Sized> OutputType for Box<T> {
 
 #[async_trait::async_trait]
 impl<T: InputType> InputType for Box<T> {
+    fn type_name() -> Cow<'static, str> {
+        T::type_name()
+    }
+
+    fn create_type_info(registry: &mut Registry) -> String {
+        T::create_type_info(registry)
+    }
+
     fn parse(value: Option<ConstValue>) -> InputValueResult<Self> {
         T::parse(value)
             .map(Box::new)
@@ -166,7 +170,8 @@ impl<T: InputType> InputType for Box<T> {
     }
 }
 
-impl<T: Type + ?Sized> Type for Arc<T> {
+#[async_trait::async_trait]
+impl<T: OutputType + ?Sized> OutputType for Arc<T> {
     fn type_name() -> Cow<'static, str> {
         T::type_name()
     }
@@ -174,10 +179,7 @@ impl<T: Type + ?Sized> Type for Arc<T> {
     fn create_type_info(registry: &mut Registry) -> String {
         T::create_type_info(registry)
     }
-}
 
-#[async_trait::async_trait]
-impl<T: OutputType + ?Sized> OutputType for Arc<T> {
     #[allow(clippy::trivially_copy_pass_by_ref)]
     async fn resolve(
         &self,
@@ -189,6 +191,14 @@ impl<T: OutputType + ?Sized> OutputType for Arc<T> {
 }
 
 impl<T: InputType> InputType for Arc<T> {
+    fn type_name() -> Cow<'static, str> {
+        T::type_name()
+    }
+
+    fn create_type_info(registry: &mut Registry) -> String {
+        T::create_type_info(registry)
+    }
+
     fn parse(value: Option<ConstValue>) -> InputValueResult<Self> {
         T::parse(value)
             .map(Arc::new)
