@@ -3,7 +3,6 @@ mod export_sdl;
 mod stringify_exec_doc;
 
 use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
-use std::sync::Arc;
 
 use indexmap::map::IndexMap;
 use indexmap::set::IndexSet;
@@ -11,8 +10,10 @@ use indexmap::set::IndexSet;
 use crate::parser::types::{
     BaseType as ParsedBaseType, Field, Type as ParsedType, VariableDefinition,
 };
-use crate::validators::InputValueValidator;
-use crate::{model, Any, Context, Positioned, ServerResult, Type, Value, VisitorContext};
+use crate::{
+    model, Any, Context, InputType, OutputType, Positioned, ServerResult, SubscriptionType, Value,
+    VisitorContext,
+};
 
 pub use cache_control::CacheControl;
 
@@ -106,7 +107,6 @@ pub struct MetaInputValue {
     pub description: Option<&'static str>,
     pub ty: String,
     pub default_value: Option<String>,
-    pub validator: Option<Arc<dyn InputValueValidator>>,
     pub visible: Option<MetaVisibleFn>,
     pub is_secret: bool,
 }
@@ -174,12 +174,6 @@ pub struct MetaEnumValue {
     pub visible: Option<MetaVisibleFn>,
 }
 
-#[derive(Clone)]
-pub struct MetaUnionValue {
-    pub name: String,
-    pub visible: Option<MetaVisibleFn>,
-}
-
 type MetaVisibleFn = fn(&Context<'_>) -> bool;
 
 #[derive(Clone)]
@@ -215,7 +209,6 @@ pub enum MetaType {
     Union {
         name: String,
         description: Option<&'static str>,
-        union_values: IndexMap<String, MetaUnionValue>,
         possible_types: IndexSet<String>,
         visible: Option<MetaVisibleFn>,
         rust_typename: &'static str,
@@ -261,10 +254,7 @@ impl MetaType {
             MetaType::Enum { visible, .. } => visible,
             MetaType::InputObject { visible, .. } => visible,
         };
-        match visible {
-            Some(f) => f(ctx),
-            None => true,
-        }
+        is_visible(ctx, visible)
     }
 
     #[inline]
@@ -375,15 +365,41 @@ pub struct Registry {
 }
 
 impl Registry {
-    pub fn create_type<T: crate::Type + ?Sized, F: FnMut(&mut Registry) -> MetaType>(
+    pub fn create_input_type<T: InputType + ?Sized, F: FnMut(&mut Registry) -> MetaType>(
         &mut self,
         mut f: F,
     ) -> String {
-        let name = T::type_name();
+        self.create_type(&mut f, &*T::type_name(), std::any::type_name::<T>());
+        T::qualified_type_name()
+    }
 
-        match self.types.get(name.as_ref()) {
+    pub fn create_output_type<T: OutputType + ?Sized, F: FnMut(&mut Registry) -> MetaType>(
+        &mut self,
+        mut f: F,
+    ) -> String {
+        self.create_type(&mut f, &*T::type_name(), std::any::type_name::<T>());
+        T::qualified_type_name()
+    }
+
+    pub fn create_subscription_type<
+        T: SubscriptionType + ?Sized,
+        F: FnMut(&mut Registry) -> MetaType,
+    >(
+        &mut self,
+        mut f: F,
+    ) -> String {
+        self.create_type(&mut f, &*T::type_name(), std::any::type_name::<T>());
+        T::qualified_type_name()
+    }
+
+    fn create_type<F: FnMut(&mut Registry) -> MetaType>(
+        &mut self,
+        f: &mut F,
+        name: &str,
+        rust_typename: &str,
+    ) {
+        match self.types.get(name) {
             Some(ty) => {
-                let rust_typename = std::any::type_name::<T>();
                 if let Some(prev_typename) = ty.rust_typename() {
                     if prev_typename != "__fake_type__" && rust_typename != prev_typename {
                         panic!(
@@ -396,7 +412,7 @@ impl Registry {
             None => {
                 // Inserting a fake type before calling the function allows recursive types to exist.
                 self.types.insert(
-                    name.clone().into_owned(),
+                    name.to_string(),
                     MetaType::Object {
                         name: "".to_string(),
                         description: None,
@@ -413,11 +429,25 @@ impl Registry {
                 *self.types.get_mut(&*name).unwrap() = ty;
             }
         }
-
-        T::qualified_type_name()
     }
 
-    pub fn create_dummy_type<T: Type>(&mut self) -> MetaType {
+    pub fn create_fake_output_type<T: OutputType>(&mut self) -> MetaType {
+        T::create_type_info(self);
+        self.types
+            .get(&*T::type_name())
+            .cloned()
+            .expect("You definitely encountered a bug!")
+    }
+
+    pub fn create_fake_input_type<T: InputType>(&mut self) -> MetaType {
+        T::create_type_info(self);
+        self.types
+            .get(&*T::type_name())
+            .cloned()
+            .expect("You definitely encountered a bug!")
+    }
+
+    pub fn create_fake_subscription_type<T: SubscriptionType>(&mut self) -> MetaType {
         T::create_type_info(self);
         self.types
             .get(&*T::type_name())
@@ -503,19 +533,6 @@ impl Registry {
             MetaType::Union {
                 name: "_Entity".to_string(),
                 description: None,
-                union_values: possible_types
-                    .clone()
-                    .into_iter()
-                    .map(|ty| {
-                        (
-                            ty.clone(),
-                            MetaUnionValue {
-                                name: ty,
-                                visible: None,
-                            },
-                        )
-                    })
-                    .collect(),
                 possible_types,
                 visible: None,
                 rust_typename: "async_graphql::federation::Entity",
@@ -524,7 +541,7 @@ impl Registry {
     }
 
     pub(crate) fn create_federation_types(&mut self) {
-        Any::create_type_info(self);
+        <Any as InputType>::create_type_info(self);
 
         self.types.insert(
             "_Service".to_string(),
@@ -595,7 +612,6 @@ impl Registry {
                                 description: None,
                                 ty: "[_Any!]!".to_string(),
                                 default_value: None,
-                                validator: None,
                                 visible: None,
                                 is_secret: false,
                             },
@@ -659,8 +675,8 @@ impl Registry {
         names.into_iter().collect()
     }
 
-    pub fn set_description<T: Type>(&mut self, desc: &'static str) {
-        match self.types.get_mut(&*T::type_name()) {
+    pub fn set_description(&mut self, name: &str, desc: &'static str) {
+        match self.types.get_mut(name) {
             Some(MetaType::Scalar { description, .. }) => *description = Some(desc),
             Some(MetaType::Object { description, .. }) => *description = Some(desc),
             Some(MetaType::Interface { description, .. }) => *description = Some(desc),
@@ -766,18 +782,6 @@ impl Registry {
             traverse_type(&self.types, &mut used_types, ty.name());
         }
 
-        fn is_system_type(name: &str) -> bool {
-            if name.starts_with("__") {
-                return true;
-            }
-
-            name == "Boolean"
-                || name == "Int"
-                || name == "Float"
-                || name == "String"
-                || name == "ID"
-        }
-
         for ty in self.types.values() {
             let name = ty.name();
             if !is_system_type(name) && !used_types.contains(name) {
@@ -789,4 +793,143 @@ impl Registry {
             self.types.remove(&type_name);
         }
     }
+
+    pub fn find_visible_types(&self, ctx: &Context<'_>) -> HashSet<&str> {
+        let mut visible_types = HashSet::new();
+
+        fn traverse_field<'a>(
+            ctx: &Context<'_>,
+            types: &'a BTreeMap<String, MetaType>,
+            visible_types: &mut HashSet<&'a str>,
+            field: &'a MetaField,
+        ) {
+            if !is_visible(ctx, &field.visible) {
+                return;
+            }
+
+            traverse_type(
+                ctx,
+                types,
+                visible_types,
+                MetaTypeName::concrete_typename(&field.ty),
+            );
+            for arg in field.args.values() {
+                traverse_input_value(ctx, types, visible_types, arg);
+            }
+        }
+
+        fn traverse_input_value<'a>(
+            ctx: &Context<'_>,
+            types: &'a BTreeMap<String, MetaType>,
+            visible_types: &mut HashSet<&'a str>,
+            input_value: &'a MetaInputValue,
+        ) {
+            if !is_visible(ctx, &input_value.visible) {
+                return;
+            }
+
+            traverse_type(
+                ctx,
+                types,
+                visible_types,
+                MetaTypeName::concrete_typename(&input_value.ty),
+            );
+        }
+
+        fn traverse_type<'a>(
+            ctx: &Context<'_>,
+            types: &'a BTreeMap<String, MetaType>,
+            visible_types: &mut HashSet<&'a str>,
+            type_name: &'a str,
+        ) {
+            if visible_types.contains(type_name) {
+                return;
+            }
+
+            if let Some(ty) = types.get(type_name) {
+                if !ty.is_visible(ctx) {
+                    return;
+                }
+
+                visible_types.insert(type_name);
+                match ty {
+                    MetaType::Object { fields, .. } => {
+                        for field in fields.values() {
+                            traverse_field(ctx, types, visible_types, field);
+                        }
+                    }
+                    MetaType::Interface {
+                        fields,
+                        possible_types,
+                        ..
+                    } => {
+                        for field in fields.values() {
+                            traverse_field(ctx, types, visible_types, field);
+                        }
+                        for type_name in possible_types.iter() {
+                            traverse_type(ctx, types, visible_types, type_name);
+                        }
+                    }
+                    MetaType::Union { possible_types, .. } => {
+                        for type_name in possible_types.iter() {
+                            traverse_type(ctx, types, visible_types, type_name);
+                        }
+                    }
+                    MetaType::InputObject { input_fields, .. } => {
+                        for field in input_fields.values() {
+                            traverse_input_value(ctx, types, visible_types, field);
+                        }
+                    }
+                    _ => {}
+                }
+            }
+        }
+
+        for type_name in Some(&self.query_type)
+            .into_iter()
+            .chain(self.mutation_type.iter())
+            .chain(self.subscription_type.iter())
+        {
+            traverse_type(ctx, &self.types, &mut visible_types, type_name);
+        }
+
+        for ty in self.types.values().filter(|ty| match ty {
+            MetaType::Object {
+                keys: Some(keys), ..
+            }
+            | MetaType::Interface {
+                keys: Some(keys), ..
+            } => !keys.is_empty(),
+            _ => false,
+        }) {
+            traverse_type(ctx, &self.types, &mut visible_types, ty.name());
+        }
+
+        self.types
+            .values()
+            .filter_map(|ty| {
+                let name = ty.name();
+                if is_system_type(name) || visible_types.contains(name) {
+                    Some(name)
+                } else {
+                    None
+                }
+            })
+            .collect()
+    }
+}
+
+pub(crate) fn is_visible(ctx: &Context<'_>, visible: &Option<MetaVisibleFn>) -> bool {
+    match visible {
+        Some(f) => f(ctx),
+        None => true,
+    }
+}
+
+fn is_system_type(name: &str) -> bool {
+    if name.starts_with("__") {
+        return true;
+    }
+
+    name == "Boolean" || name == "Int" || name == "Float" || name == "String" || name == "ID"
 }
