@@ -55,20 +55,7 @@ impl<'a> Visitor<'a> for ArgumentsOfCorrectType<'a> {
                 })
                 .ok();
 
-            if let Some(validator) = &arg.validator {
-                if let Some(value) = &value {
-                    if let Err(e) = validator.is_valid_with_extensions(value) {
-                        ctx.report_error_with_extensions(
-                            vec![name.pos],
-                            format!("Invalid value for argument \"{}\", {}", arg.name, e.message),
-                            e.extensions,
-                        );
-                        return;
-                    }
-                }
-            }
-
-            if let Some(e) = value.and_then(|value| {
+            if let Some(reason) = value.and_then(|value| {
                 is_valid_input_value(
                     ctx.registry,
                     &arg.ty,
@@ -79,10 +66,9 @@ impl<'a> Visitor<'a> for ArgumentsOfCorrectType<'a> {
                     },
                 )
             }) {
-                ctx.report_error_with_extensions(
+                ctx.report_error(
                     vec![name.pos],
-                    format!("Invalid value for argument {}", e.message),
-                    e.extensions,
+                    format!("Invalid value for argument {}", reason),
                 );
             }
         }
